// =============== Klassen-Definition: MATRIZEN  ========================================
// MATRIZEN-Basis-Klasse fuer allgemeine Matrizen vom Typ double
// Matrizen mat(i,j,k,l) werden spaltenweise gespeichert !!!
// Indizierung von i,j,k,l beginnt bei 0 !!!
// vgl. U. Breymann, C++, Carl Hanser Verlag 1993, S.186 ff
// ======================================================================================
// Copyright: Univ.-Prof. Dr.-Ing. Udo F. Meissner, Erkrath/Darmstadt
// experimentelles Lehr- und Studienprogramm - kommerzielle Nutzung nicht gestattet
// Der Autor bernimmt keine Haftung fr Unvollstndigkeiten und Maengel der Algorithmen!
// Version:  6/1995
// Compiler: C++
// ================ Include-Files =======================================================
#include "pch.h"
#include "framework.h"
#include "MATRIXclass.h"
// ---------------------
// #include "Matrizen.h"
// ============================================================================
// public -> Oeffentliche Mitgliedsfunktionen
// ============================================================================
//
// Matrix gegen Loeschen sperren -> mat.sperren()
// ==============================================
void MATRIZEN::sperren()
{
// permanente Matrix
if(typ == 1) typ=2;
};//--------------------------------------------------------------------------
//
// Matrix fuer Loeschen entsperren -> mat.entsperren()
// ===================================================
void MATRIZEN::entsperren()
{
// permanente Matrix
if(typ == 2) typ=1;
};//--------------------------------------------------------------------------
//
// WERT suchen
// ===========
// Argumente:
// knonr: Wert, nach dem gesucht wird
// ni,nj,nk,nl    : Dimensionslaengen der Matrix
//                : -1 Laenge automatisch ermitteln 
// di,dj,dk,dl    : Schrittweite, mit der die Matrix durchsucht wird
// Funktionsergebnis:  1 knonr wurde gefunden
//                    -1 knonr wurde nicht gefunden
int MATRIZEN::wert_suchen(double knonr, int &i,int &j,int &k,int &l,
                     int ni,int di,int nj,int dj,int nk,int dk,int nl,int dl)
{
// Dimension ermitteln
int ii,jj,kk,ll,zi,zj,zk,zl;
//
this->dimension_ermitteln(zi,zj,zk,zl);
//
if(ni >= 0) zi=ni;
if(nj >= 0) zj=nj;
if(nk >= 0) zk=nk;
if(nl >= 0) zl=nl;  
//
int wert_intern=-1;
//
for(ll=0; ll<zl/dl; ll+=dl)
for(kk=0; kk<zk/dk; kk+=dk)
for(jj=0; jj<zj/dj; jj+=dj)
for(ii=0; ii<zi/di; ii+=di)
{
if(this->element(ii,jj,kk,ll) == knonr) {wert_intern=1; goto Ende;};
};
ii=jj=kk=ll=-1;
//
Ende:
i=ii; j=jj; k=kk; l=ll;
return wert_intern;
};
// ===========================================================================
// public -> Oeffentliche, virtuelle Mitgliedsfunktionen
// ===========================================================================
//
// virtueller Destruktor -> delete &mat
// ====================================
MATRIZEN::~MATRIZEN(void)
{
//*** std::cerr<<"MATRIZEN::Destruktor: Matrix loeschen"<<'\n';
};//--------------------------------------------------------------------------
//============================================================================
// public -> Oeffentliche Mitgliedsfunktionen (mit temp. Matrizen !!!)
// ===========================================================================
//
// Typwandlungsoperator -> zahl=double(mat) (nur erstes Matrixelement zuweisen)
// ========================================
MATRIZEN::operator double ()
{
double zahl;
zahl=*(this->matrix);
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return zahl;
};//--------------------------------------------------------------------
//
// Matrixindex -> (i)
// ==================
double &MATRIZEN::operator () (int i)
{
// aktuelles Matrixelement notieren
aktuelles_element=this->adresse(i);
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *aktuelles_element;
};//--------------------------------------------------------------------
// Matrixindex -> (i,j)
// ====================
double &MATRIZEN::operator () (int i,int j)
{
// aktuelles Matrixelement notieren
aktuelles_element=this->adresse(i,j);
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *aktuelles_element;
};//--------------------------------------------------------------------
// Matrixindex -> (i,j,k)
// ======================
double &MATRIZEN::operator () (int i,int j,int k)
{
// aktuelles Matrixelement notieren
aktuelles_element=this->adresse(i,j,k);
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *aktuelles_element;
};//--------------------------------------------------------------------
//
// Matrixindex -> (i,j,k,l)
// ========================
double &MATRIZEN::operator () (int i,int j,int k,int l)
{
// aktuelles Matrixelement notieren
aktuelles_element=this->adresse(i,j,k,l);
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *aktuelles_element;
};//--------------------------------------------------------------------------
//
// Matrizenelement -> mat.element(i,j,k,l) vergleiche mat(i,j,k,l)
// =============================================================
double &MATRIZEN::element(int i,int j,int k,int l)
{
aktuelles_element=this->adresse(i,j,k,l);
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *aktuelles_element;
};//--------------------------------------------------------------------------
//
// Aktuelles Matrixelement holen und den Elementzeiger inkrementieren -> mat++ 
// ===========================================================================
// Durch den expliziten Aufruf -> mat.operator ++ (abstand)
//         kann der Elementzeiger auch um mehrere Elemente inkrementiert werden
// Der Elementzeiger wird nur vom Konstruktor, vom Destruktor
//                 und von den Operatoren operator (), operator -- veraendert !
// ----------------------------------------------------------------------------
double &MATRIZEN::operator ++ (int abstand)
{
// Der Aufruf mat++ erfolgt ueber mat.operator ++ (0) -> Stroustrup, S. 338
if(abstand == 0) abstand=1;
//
// Aktuelles Matrixelement
double *matrix_aktuell=aktuelles_element;
//
// aktuelles Matrixelement inkrementieren
aktuelles_element+=abstand;
// Grenzen ueberpruefen
if(aktuelles_element < matrix)     aktuelles_element=matrix;
double *matrix_max=matrix+dimmax-1;
if(aktuelles_element > matrix_max) aktuelles_element=matrix_max;
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *matrix_aktuell;
};//--------------------------------------------------------------------
//
// Aktuelles Matrixelement holen und den Elementzeiger dekrementieren -> mat-- 
// ===========================================================================
// Durch den expliziten Aufruf -> mat.operator -- (abstand)
//         kann der Elementzeiger auch um mehrere Elemente dekrementiert werden
// Der Elementzeiger wird nur vom Konstruktor, vom Destruktor
//                 und von den Operatoren operator (), operator ++ veraendert !
// ----------------------------------------------------------------------------
double &MATRIZEN::operator -- (int abstand)
{
// Der Aufruf mat-- erfolgt ueber mat.operator -- (0) -> Stroustrup, S. 338
if(abstand == 0) abstand=1;
//
// Aktuelles Matrixelement
double *matrix_aktuell=aktuelles_element;
//
// aktuelles Matrixelement dekrementieren
aktuelles_element=aktuelles_element-abstand;
// Grenzen ueberpruefen
if(aktuelles_element < matrix)     aktuelles_element=matrix;
double *matrix_max=matrix+dimmax-1;
if(aktuelles_element > matrix_max) aktuelles_element=matrix_max;
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *matrix_aktuell;
};//--------------------------------------------------------------------
//
// Matrix-Dimensionen temporaer aendern -> mat.dimension(i,j,k,l)
// ==============================================================
MATRIZEN &MATRIZEN::dimension(int ni,int nj,int nk,int nl)
{
// Zwischenspeicher temp
MATRIZEN *temp=temporaere_matrix_erzeugen(*this,0);
// -----------------------------------------------
temp->dimension_temporaer(ni,nj,nk,nl);
// -----------------------------------------------
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *temp;
};//--------------------------------------------------------------------
//
// Matrix-Dimensionen ermitteln -> mat.dimension_ermitteln(ni,nj,nk,nl)
// ====================================================================
// Ergebnis: dim=Anzahl der Matrixelemente (belegter Speicherplatz)
int MATRIZEN::dimension_ermitteln(int &ni,int &nj,int &nk,int &nl)
{
ni=dimi; nj=dimj; nk=dimk; nl=diml;
//
return dimmax;
};//--------------------------------------------------------------------
//
// Matrix-Dimensionen permanent aendern -> mat.dimension_aendern(i,j,k,l)
// ======================================================================
MATRIZEN &MATRIZEN::dimension_aendern(int ni,int nj,int nk,int nl)
{
int max=ni*nj*nk*nl;
if(max > dimmax)
{
 std::cout<<"*** Fehler in Funktion MATRIZEN::dimension_aendern -> Falsche Indizes:"<<'\n';
 std::cout<<"dimmax="<<dimmax<<'\n';
 std::cout<<"dimi="<<dimi<<" dimj="<<dimj<<" dimk="<<dimk<<" diml="<<diml<<'\n';
 std::cout<<"  ni="<<  ni<<"   nj="<<  nj<<"   nk="<<  nk<<"   nl="<<  nl<<'\n';
 std::cout<<"max   ="<<max<<'\n';
char c;
 std::cin>>c;
};
// -----------------------------------
// Dimensionen aendern
dimi=ni;
dimj=nj;
dimk=nk;
diml=nl;
dimmax=max;
// -----------------------------------
return *this;    
};//--------------------------------------------------------------------
//
// Anzahl der Zeilen/Spalten ermitteln -> mat.zeilen_spalten(ni,nj,nk,nl,dim)
// ==========================================================================
// Ergebnis: Anzahl der Zeilen/Spalten=sqrt(ni*nj*nk*nl)
//           von quadratischen Matrizen,
//           sonst -1
int MATRIZEN::zeilen_spalten(int &ni,int &nj,int &nk,int &nl)
{
ni=dimi; nj=dimj; nk=dimk; nl=diml;
// Zeilen/Spalten von quadratischen Matrizen
int zeilen=dimi*dimj*dimk*diml;
double spalten=double(zeilen)+0.5;
spalten=sqrt(spalten);
int z=int(spalten);
//
if(z*z != zeilen) z=-1;
//
return z;
};//--------------------------------------------------------------------
//
// Teilmatrix heraus -> ...=mat.teilmatrix_heraus(i,j, ni,nj, k,l, nk,nl)
// ======================================================================
MATRIZEN &MATRIZEN::teilmatrix_heraus(int i,int j, int ni,int nj,
                                      int k,int l, int nk,int nl)
{
// Zwischenspeicher temp
MATRIZEN *temp=temporaere_matrix_erzeugen(ni,nj,nk,nl);
// ----------------------------------------------------
temp->teil_heraus(*this,i,j,k,l);
// --------------------------------------------------
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *temp;
};//--------------------------------------------------------------------
//
// Teilmatrix hinein -> mat.teilmatrix_hinein(i,j,k,l)=...
// =======================================================
MATRIZEN &MATRIZEN::teilmatrix_hinein(int i,int j,int k,int l)
{
// Zwischenspeicher temp
MATRIZEN *temp=temporaere_matrix_erzeugen(*this,0);
// ------------------------------------------------
temp->teil_hinein(i,j,k,l);
// ------------------------------------------------
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *temp;
};//--------------------------------------------------------------------
//
// Untermatrix heraus -> ...=mat.untermatrix_heraus(i,ni, j,nj, k,nk, l,nl)
// ========================================================================
MATRIZEN &MATRIZEN::untermatrix_heraus(int i,int ni, int j,int nj,
                                       int k,int nk, int l,int nl)
{
// Zwischenspeicher temp
MATRIZEN *temp=temporaere_matrix_erzeugen(ni,nj,nk,nl);
//-----------------------------------------------------
temp->unter_heraus(*this,i,j,k,l);
//-----------------------------------------------------
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *temp;
};//--------------------------------------------------------------------
//
// Untermatrix hinein -> mat.untermatrix_hinein(i,j,k,l)=...
// =========================================================
MATRIZEN &MATRIZEN::untermatrix_hinein(int i,int j,int k,int l)
{
// Zwischenspeicher temp
MATRIZEN *temp=temporaere_matrix_erzeugen(*this,0);
//-------------------------------------------------
temp->unter_hinein(i,j,k,l);
//-------------------------------------------------
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *temp;
};//--------------------------------------------------------------------
//
// Matrix-Ausgabe -> cout<<mat
// ===========================
std::ostream& operator << (std::ostream& s,MATRIZEN& mat)
{
char c;
long alt_flags;
//*** int
long long alt_ziffern;
//
if(&s == &std::cout)
{
    // alte Zahlengenauigkeit retten
    // -----------------------------
    alt_flags = std::cout.flags();
    alt_ziffern = std::cout.precision();
    //
  // neue Zahlengenauigkeit setzen
  // -----------------------------
    std::cout.setf(ios::scientific);
  //*** STD::cout.setf(ios::fixed);
    std::cout.precision(3);
};
// -------------------------------------
mat.ausgeben(s);
// -------------------------------------
if(&s == &std::cout)
{
  // alte Zahlengenauigkeit
  // ----------------------
  std::cout.flags(alt_flags);
  std::cout.precision(alt_ziffern);
  // Pause
  // -----
    std::cout << "Pause -> Zeichen eingeben:";
 // std::cout << "Pause -> Enter a character:";
  // bel
 std::cout<<'\07';
  //
  // Abfrage auf return (Zeichen 10 = LF, line feed = Zeilenvorschub)
  int ret= std::cin.get(); ret= std::cin.peek();
  if(ret != 10) std::cin>>c;
};
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
mat.temporaere_matrix_loeschen();
//
return s;
};//---------------------------------------------------------------------------
//
// Matrix-Eingabe -> cin>>mat
// ==========================
std::istream &operator >> (std::istream &s,MATRIZEN &mat)
{
// --------------------------------
 mat.einlesen(s);
// --------------------------------
if(&s == &std::cin) {std::cout<<"Eingabe fertig !"<<'\n';};
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
mat.temporaere_matrix_loeschen();
//
return s;
};//--------------------------------------------------------------------
//
// Alle Matrix-Elemente belegen -> mat =zahl
// =========================================
MATRIZEN &MATRIZEN::operator = (double zahl)
{
this->zuweisen(zahl,0);
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *this;
};//--------------------------------------------------------------------
//
// Alle Matrix-Elemente belegen -> mat+=zahl
// =========================================
MATRIZEN &MATRIZEN::operator += (double zahl)
{
this->zuweisen(zahl,2);
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *this;
};//--------------------------------------------------------------------
//
// Alle Matrix-Elemente belegen -> mat-=zahl
// =========================================
MATRIZEN &MATRIZEN::operator -= (double zahl)
{
this->zuweisen(zahl,4);
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *this;
};//--------------------------------------------------------------------
//
// Matrix zuweisen -> mat1 =mat2 (nur Matrizeninhalt kopieren)
// =============================
MATRIZEN &MATRIZEN::operator  = (MATRIZEN &mat2)
{
this->zuweisen(mat2,0);
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
mat2.temporaere_matrix_loeschen();
//
return *this;
};//---------------------------------------------------------------------------
//
// Matrix zuweisen -> mat1+=mat2 (nur Matrizeninhalt kopieren)
// =============================
MATRIZEN &MATRIZEN::operator += (MATRIZEN &mat2)
{
this->zuweisen(mat2,2);
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
mat2.temporaere_matrix_loeschen();
//
return *this;
};//--------------------------------------------------------------------
//
// Matrix zuweisen -> mat1-=mat2 (nur Matrizeninhalt kopieren)
// =============================
MATRIZEN &MATRIZEN::operator -= (MATRIZEN &mat2)
{
this->zuweisen(mat2,4);
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
mat2.temporaere_matrix_loeschen();
//
return *this;
};//--------------------------------------------------------------------
//
// positives Vorzeichen -> +mat
// ============================
MATRIZEN &MATRIZEN::operator + ()
{
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *this;
};//--------------------------------------------------------------------
//
// Addition mit Skalar -> mat+zahl
// ===============================
MATRIZEN &MATRIZEN::operator + (double zahl)
{
// Zwischenspeicher temp
MATRIZEN *temp=temporaere_matrix_erzeugen(*this,-1);
// -------------------------------------------------
for (int i=0; i<dimmax; i++)
{
*(temp->matrix+i)=*(matrix+i)+zahl;
};
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *temp;
};//--------------------------------------------------------------------
//
// Addition mit Skalar -> zahl+mat
// ===============================
MATRIZEN &operator + (double zahl,MATRIZEN &mat)
{
return mat+zahl;
};//--------------------------------------------------------------------
//
// Addition -> mat1+mat2 bzw. mat1+vek2 bzw. vek1+mat2
// ===================================================
MATRIZEN &MATRIZEN::operator + (MATRIZEN &mat2)
{
// Zwischenspeicher temp
MATRIZEN *temp;
// ------------ 
//
if (mat2.dimmax > this->dimmax)
temp=temporaere_matrix_erzeugen(mat2,-1);
// --------------------------------------
else
temp=temporaere_matrix_erzeugen(*this,-1);
// ---------------------------------------
//
temp->addieren(*this,mat2);
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
mat2.temporaere_matrix_loeschen();
//
return *temp;
};//---------------------------------------------------------------------------
//
// negatives Vorzeichen -> -mat
// ============================
MATRIZEN &MATRIZEN::operator - ()
{
// Zwischenspeicher temp
MATRIZEN *temp=temporaere_matrix_erzeugen(*this,-1);
// -------------------------------------------------
for (int i=0; i<dimmax; i++)
{
*(temp->matrix+i)=-(*(matrix+i));
};
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *temp;
};//--------------------------------------------------------------------
//
// Subtraktion mit Skalar -> mat-zahl
// ==================================
MATRIZEN &MATRIZEN::operator - (double zahl)
{
// Zwischenspeicher temp
MATRIZEN *temp=temporaere_matrix_erzeugen(*this,-1);
// -------------------------------------------------
for (int i=0; i<dimmax; i++)
{
*(temp->matrix+i)=*(matrix+i)-zahl;
};
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *temp;
};//--------------------------------------------------------------------
//
// Subtraktion mit Skalar -> zahl-mat
// ==================================
MATRIZEN &operator - (double zahl,MATRIZEN &mat)
{
return -mat+zahl;
};//--------------------------------------------------------------------
//
// Subtraktion -> mat1-mat2 bzw. mat1-vek2 bzw. vek1-mat2
// ======================================================
MATRIZEN &MATRIZEN::operator - (MATRIZEN &mat2)
{
// Zwischenspeicher temp
MATRIZEN *temp;
// ------------ 
if (mat2.dimmax > this->dimmax)
temp=temporaere_matrix_erzeugen(mat2,-1);
// --------------------------------------
else
temp=temporaere_matrix_erzeugen(*this,-1);
// ---------------------------------------
//
temp->subtrahieren(*this,mat2);
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
mat2.temporaere_matrix_loeschen();
//
return *temp;
};//---------------------------------------------------------------------------
//
// Multiplikation mit Skalar -> mat*zahl
// =====================================
MATRIZEN &MATRIZEN::operator * (double zahl)
{
//*** std::cerr<<"MATRIX:: mat*zahl"<<'\n';
// Zwischenspeicher temp
MATRIZEN *temp=temporaere_matrix_erzeugen(*this,-1);
// -------------------------------------------------
for (int i=0; i<dimmax; i++)
{
*(temp->matrix+i)=*(matrix+i)*zahl;
};
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return *temp;
};//--------------------------------------------------------------------
//
// Multiplikation mit Skalar -> zahl*mat
// =====================================
MATRIZEN &operator * (double zahl,MATRIZEN &mat)
{
//*** std::cerr<<"MATRIX::zahl*mat"<<'\n';
return mat*zahl;
};//--------------------------------------------------------------------
//
// Division mit Skalar -> mat/zahl
// ===============================
MATRIZEN &MATRIZEN::operator / (double zahl)
{
double z=1.0/zahl;
return (*this)*z;
};//-------------------------------------------------------------------------------------
//
// Matrizenmultiplikation -> matc = mat1 * mat2 -> matc(i,j,k,l) = mat1(i,s,j) * mat2(k,s,l)
// =========================================================================================
MATRIZEN &MATRIZEN::operator * (MATRIZEN &mat2)
{
// Matrixdimensionen ermitteln
int ni, ns, nj, nk, nss, nl, dummy;
this->dimension_ermitteln(ni, ns,  nj, dummy);
 mat2.dimension_ermitteln(nk, nss, nl, dummy);
// 
//*** std::cout << "MATRIZEN: operator *" << endl;
//*** std::cout << "MATRIZEN::(*this) mat1(i,s,j) * mat2(k,s,l) ni, ns, nj, nk, nss, nl = " <<ni<<ns<<nj<<nk<<nss<<nl<< endl;
//
if (ns != nss) 
    { std::cout << "MATRIZEN::multiplizieren: Fehler => mat1(" <<ni<< ", " <<ns<< " , " <<nj<<") * mat2(" <<nk<< ", " <<nss<< " , "<<nl<< ")" <<endl;
      std::cout << "MATRIZen::multiplizieren: Fehler => Dimensionen nicht kompatibel !!!" << endl;
    }
//
// Zwischenspeicher temp
MATRIZEN *temp=temporaere_matrix_erzeugen(ni,nj,nk,nl);
// ----------------------------------------------------
// Matrizen multiplizieren
temp->multiplizieren(*this,mat2,0);
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
mat2.temporaere_matrix_loeschen();
//
return *temp;
};//---------------------------------------------------------------------------
//
// Matrizendivision    matx(s,n) = matr(i,n) / MATk(i,s)
//         MATk(i,s) * matx(s,n) = matr(i,n)
// =====================================================
MATRIZEN& MATRIZEN::operator / (MATRIZEN& MATk)
{
// Matrixdimensionen ermitteln
int ni, nn, nii, ns, dummy;
 MATk.dimension_ermitteln(ni,  ns, dummy, dummy);
this->dimension_ermitteln(nii, nn, dummy, dummy);
// 
//*** std::cout << "MATRIZEN: operator /" << endl;
//*** std::cout << "MATRIZEN::(*this) matr(i,n) / MATk(i,s) ni, nn, nii, ns = " << ni << nn << nii << ns << '\n';
// 
if (ni != nii)
   {
    std::cout << "MATRIZEN::dividieren: Fehler => matr(" << nii << ", " << nn << ") * MATk(" << ni << " , " << ns << ")" << endl;
    std::cout << "MATRIZen::dividieren: Fehler => Dimensionen nicht kompatibel !!!" << endl;
   }
//
// Zwischenspeicher temp
MATRIZEN* temp = temporaere_matrix_erzeugen(*this, 1);
// ----------------------------------------------------
// Gleichungssystem lsen
// ----------------------
*temp = MATk.dividieren(*this);
//
//*** std::cout << "MATRIZEN::operator /: *temp= " << *temp << endl;
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
MATk.temporaere_matrix_loeschen();
this->temporaere_matrix_loeschen();
//
return *temp;
};//--------------------------------------------------------------------------
//
// Betrag einer Matrix -> mat.betrag()
// ===================================
double MATRIZEN::betrag()
{
double sum=skalarprodukt(matrix,1,matrix,1,dimmax);
//
// ggfls. temporaere Matrix loeschen, Original-Deskriptor wiederherstellen
this->temporaere_matrix_loeschen();
//
return sqrt(sum);
};//---------------------------------------------------------------------------
//
// Matrix normieren -> mat.normieren()
// ===================================
MATRIZEN &MATRIZEN::normieren()
{
double betrag;
//
return this->normieren(betrag);
};//--------------------------------------------------------------------------
// Matrix normieren -> mat.normieren(betrag)
// =========================================
MATRIZEN &MATRIZEN::normieren(double &betrag_)
{
// Zwischenspeicher temp
MATRIZEN *temp=temporaere_matrix_erzeugen(*this,-1);
// -------------------------------------------------
double wert,*temp_vektor,*vektor;
// Betrag des Vektors
betrag_=this->betrag();
wert=1.0/betrag_;
//
temp_vektor=temp->matrix;
//
for (vektor=matrix; vektor<matrix+dimmax; vektor++)
{
*(temp_vektor++)=*vektor*wert;
};
// ggfls. temporaere Matrix loeschen
this->temporaere_matrix_loeschen();
//
return *temp;
};//---------------------------------------------------------------------------
//
// ============================================================================
// protected -> Geschuetzte Mitgliedsfunktionen
// ============================================================================
//
// Skalarprodukt -> a*b -> zahl= a(s) * b(s)
// =========================================
double MATRIZEN::skalarprodukt(double *a,int da,double *b,int db,int n)
{
double *ea=a+da*n-da;
double sum=0.0;
// ---------------
if(da >= 0)
{
  for(a=a;a<=ea;a+=da)
  {
    sum+=(*a)*(*b);
    b+=db;
  };
}
else
{
  for(a=a;a>=ea;a+=da)
  {
    sum+=(*a)*(*b);
    b+=db;
  };
};
//
return sum;
};
// ============================================================================
